/***************************************************************************
 *
 * ConfigFile.cpp: a simple configfile
 *
 * (c) 2004 Thomas Unterthiner
 *
 * created:      23. May 2004
 * last change:  $Date: 2004/05/26 16:03:22 $
 *
 ***************************************************************************/
 
 
#include "ConfigFile.h"

/*
 * Ctor - reads in settings
 */
ConfigFile::ConfigFile(const string& path)
	: filepath_(path)
{
	this->readSettings();
}



/*
 * change a value, or - if the value newer existed - create it
 */
void ConfigFile::set(const string& key, const string& value)
{
	settings_[key] = value;
}



/*
 * save the actual configuration to a file.
 * if no filename was provided, save to the actual config-file
 * NOTE: due to the management in std::map, the configfile will be
 *       alphabetically sorted by descending keys;
 */
void ConfigFile::save(const string& newFilename)
{
	ofstream file;
	if (newFilename == "")
	{
		file.open(filepath_.c_str());
	}
	else
	{
		file.open(newFilename.c_str());
	}
	
	map<string, string>::iterator it = settings_.begin();
	while (it != settings_.end())
	{
		file << it->first << ASSIGN_SIGN << it->second << '\n';
		++it;
	}
}



/*
 * just like the normal get(), but the value will be copied into a C-string
 */
template<> bool ConfigFile::get<>(const string& key, char* returnValue)
{
	map<string, string>::const_iterator it = settings_.find(key);
	if (it!=settings_.end())
	{
		std::istringstream tmp(it->second);
		strcpy(returnValue, it->second.c_str() );
		return true;
	}
	return false;
}



/*
 * like the normal get, but the returnValue is a std::string, too, so
 * there's no need to cast
 */
template <> bool ConfigFile::get<>(const string& key, string* returnValue)
{
	map<string, string>::const_iterator it = settings_.find(key);
	if (it!=settings_.end())
	{
		*returnValue = it->second;
		return true;
	}
	return false;
}



/*
 * read the settings out of the file and store them in a hash
 */
void ConfigFile::readSettings()
{
	ifstream file(filepath_.c_str());
	if (!file) return;
	string key;
	string tmp;
	string value;

	while (!file.eof())
	{
		getline(file, tmp);
		if (tmp[0] == COMMENT_SIGN) continue; // jump over comments
		if (tmp[0] == '\n' || tmp[0] == '\r') continue; // jump over empty lines
		
		string::size_type keyEndIndex = tmp.find_first_of(ASSIGN_SIGN);
		if (keyEndIndex == string::npos)
		{
			#ifdef DEBUG
			if (!file.eof())
			{
				std::cerr << "WARNING: ConfigFile::readSettings() ->";
    			std::cerr << " ignoring illegal line: " << tmp << '\n';
			}
			#endif
			continue;
		}

		string::size_type valueStartIndex = keyEndIndex + 1;
		string::size_type valueEndIndex = tmp.length() -1;
		while (tmp[keyEndIndex - 1] == ' ') --keyEndIndex;
		while (tmp[valueStartIndex] == ' ') ++valueStartIndex;
		while (tmp[valueEndIndex] == ' ') --valueEndIndex;

		string key = tmp.substr(0, keyEndIndex);
		string value = tmp.substr(valueStartIndex, valueEndIndex);
		settings_[key] = value;
	}
}

