/***************************************************************************
 *
 * MorphingTable.cpp: a morphing table, where the cells change their background
 *                    color
 *
 * (c) 2004 Thomas Unterthiner
 *
 * created:      22. May 2004
 * last change:  $Date: 2004/05/31 13:42:18 $
 *
 ***************************************************************************/
 
#include "MorphingTable.h"

//#define DEBUG

/*
 * create new table of (rows*columns) size, use the given surface to 
 * draw uppon. Initiate all cells with black color
 */
MorphingTable::MorphingTable(int rows, int columns, Surface& surface)
	: rows_(rows),
	  cols_(columns),
	  cellCount_(rows_*cols_),
	  cellWidth(1),
	  cellHeight(1),
	  surface_(surface),
	  randMaxToCellsRatio_(cellCount_ / RAND_MAX)
{
	// cellWidth and cellHeight are float's, because
	// otherwhise their would be some parts of the screen empty
	// e.g. if surfaceWidth = 1024 and cols_ = 96
	// cellWidth would be 10.666666 as float, or, as int simply 10
	// if it's 10, then we're only drawing until the 960 pixel
	// (96 cols * 10 pixel per column), even if we'd have to draw
	// 1024 pixels in each row
	cellWidth = surface.getWidth() / static_cast<float>(cols_);
	cellHeight = surface.getHeight() / static_cast<float>(rows_);

	if (cellCount_ % RAND_MAX != 0)
	{
		++randMaxToCellsRatio_;
	}

	#ifdef DEBUG
		fprintf(stderr, "cols: %d\n", cols_);
		fprintf(stderr, "rows: %d\n", rows_);
		fprintf(stderr, "surfaceWidth: %d\n", surface.getWidth());
		fprintf(stderr, "surfaceHeight: %d\n", surface.getHeight());
		fprintf(stderr, "cellWidth: %f\n", cellWidth);
		fprintf(stderr, "cellHeight: %f\n", cellHeight);
	#endif
	srand(time(0));	
	for (int i = 0; i < rows_ * cols_; ++i)
	{
		cells_.push_back(Surface::Color(0, 0, 0));
	}
}


/*
 * choose a cell randomly and change its color randomly
 */
void MorphingTable::morph()
{
	int index = chooseCell();
	cells_[index].r = rand() % 255;
	cells_[index].g = rand() % 255;
	cells_[index].b = rand() % 255;
}



/*
 * draw to table on the surface
 */
void MorphingTable::draw()
{
    if (cellHeight == 1.00 && cellWidth == 1.00)
    {
        this->drawPerPixel();
        return;
    }

   	int i = 0;
   	for (float x = 0.0, y = -cellHeight; i < cells_.size(); ++i)
   	{
       	if (i % cols_ == 0)
       	{
   			y += cellHeight;
   			x = 0.0;
   		}
    
   		surface_.fillRect(static_cast<int>(x), static_cast<int>(y), 
   		                  static_cast<int>(x + cellWidth), 
   		                  static_cast<int>(y + cellHeight),
   		                  cells_[i]);
   		x += cellWidth;
   	}
	
}


/*
 * choose a cell randomly and return its index in the cells_-vector
 * because rand() returns a maximum of ca. 32k, we need another algo
 * to choose numbers... e.g. with 786432 cells (1024x768 with 1 cell
 * per pixel), most of the cells would never get choosen
 */
long MorphingTable::chooseCell()
{
	long choosen = rand() % cellCount_;
	
	if (RAND_MAX < cellCount_)
	{
		int range = rand() % randMaxToCellsRatio_;
  		choosen += range * RAND_MAX;		
	}
	
	return choosen;
}


/*
 * if cellWidth and cellHeight are 1.00 (meaning exactly 1 pixel), it's faster 
 * to draw the pixels using putPixel() instead of drawing rectangulars
 */
void MorphingTable::drawPerPixel()
{
	surface_.beforeDrawing();
   	for (int y = 0; y < rows_; ++y)
   	{
   	    for (int x = 0; x < cols_; ++x)
   	    {
   	        surface_.putPixel(x, y, cells_[y * rows_ + x]);
   	    }    
   	}
    surface_.afterDrawing();   
}

